@@ -19,6 +19,7 @@
 package org.eclipse.jetty.client.http;
 
 import java.nio.channels.AsynchronousCloseException;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -46,6 +47,8 @@ public class HttpConnectionOverHTTP extends AbstractConnection implements Connec
     private final Delegate delegate;
     private final HttpChannelOverHTTP channel;
     private long idleTimeout;
+    private int idleTimeoutGuard;
+    private long idleTimeoutStamp;
 
     /**
      * @deprecated use {@link #HttpConnectionOverHTTP(EndPoint, HttpDestination, Promise)} instead
@@ -105,13 +108,61 @@ public class HttpConnectionOverHTTP extends AbstractConnection implements Connec
         return closed.get();
     }
 
+    private void send(HttpChannelOverHTTP channel, HttpExchange exchange)
+    {
+        boolean send;
+        synchronized (this)
+        {
+            send = idleTimeoutGuard >= 0;
+            if (send)
+                ++idleTimeoutGuard;
+        }
+
+        if (send)
+        {
+            if (channel.associate(exchange))
+                channel.send();
+            else
+                channel.release();
+
+            synchronized (this)
+            {
+                --idleTimeoutGuard;
+                idleTimeoutStamp = System.nanoTime();
+            }
+        }
+        else
+        {
+            // This connection idle timed out before we could send the exchange, retry.
+            HttpDestinationOverHTTP destination = getHttpDestination();
+            destination.send(exchange);
+        }
+    }
+
     @Override
     protected boolean onReadTimeout()
     {
-        if (LOG.isDebugEnabled())
-            LOG.debug("{} idle timeout", this);
-        close(new TimeoutException());
-        return false;
+        synchronized (this)
+        {
+            if (idleTimeoutGuard == 0)
+            {
+                long elapsed = TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - idleTimeoutStamp);
+                boolean idle = elapsed > idleTimeout / 2;
+                if (LOG.isDebugEnabled())
+                    LOG.debug("Idle timeout {}/{}ms - {}", elapsed, idleTimeout, this);
+                if (idle)
+                {
+                    idleTimeoutStamp = -1;
+                    close(new TimeoutException("Idle timeout expired: " + idleTimeout + "ms"));
+                }
+            }
+            else
+            {
+                if (LOG.isDebugEnabled())
+                    LOG.debug("Idle timeout skipped - {}", this);
+            }
+            return false;
+        }
     }
 
     @Override
@@ -215,10 +266,7 @@ public class HttpConnectionOverHTTP extends AbstractConnection implements Connec
             endPoint.setIdleTimeout(request.getIdleTimeout());
 
             // One channel per connection, just delegate the send
-            if (channel.associate(exchange))
-                channel.send();
-            else
-                channel.release();
+            HttpConnectionOverHTTP.this.send(channel, exchange);
         }
 
         @Override
