@@ -21,6 +21,7 @@ package org.eclipse.jetty.client;
 import java.io.IOException;
 import java.util.Random;
 import java.util.concurrent.TimeUnit;
+
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -107,7 +108,36 @@ public class ContentResponseTest extends AbstractHttpClientServerTest
             {
                 baseRequest.setHandled(true);
                 response.setHeader(HttpHeader.CONTENT_TYPE.asString(), contentType);
-                response.getOutputStream().write(content.getBytes("UTF-8"));
+                response.getOutputStream().write(content.getBytes(encoding));
+            }
+        });
+
+        ContentResponse response = client.newRequest("localhost", connector.getLocalPort())
+                .scheme(scheme)
+                .timeout(5, TimeUnit.SECONDS)
+                .send();
+
+        Assert.assertEquals(200, response.getStatus());
+        Assert.assertEquals(content, response.getContentAsString());
+        Assert.assertEquals(mediaType, response.getMediaType());
+        Assert.assertEquals(encoding, response.getEncoding());
+    }
+
+    @Test
+    public void testResponseWithContentTypeWithQuotedCharset() throws Exception
+    {
+        final String content = "The quick brown fox jumped over the lazy dog";
+        final String mediaType = "text/plain";
+        final String encoding = "UTF-8";
+        final String contentType = mediaType + "; charset=\"" + encoding + "\"";
+        start(new AbstractHandler()
+        {
+            @Override
+            public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
+            {
+                baseRequest.setHandled(true);
+                response.setHeader(HttpHeader.CONTENT_TYPE.asString(), contentType);
+                response.getOutputStream().write(content.getBytes(encoding));
             }
         });
 
