@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2016 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2017 Mort Bay Consulting Pty. Ltd.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -24,9 +24,9 @@ import static org.junit.Assert.fail;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.OutputStream;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.nio.channels.ReadableByteChannel;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -45,13 +45,13 @@ public class OrderingTest
     public class TestResource extends Resource
     {
         public String _name;
-        
+
         public TestResource (String name)
         {
             _name =name;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#addPath(java.lang.String)
          */
         @Override
@@ -60,7 +60,7 @@ public class OrderingTest
             return null;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#delete()
          */
         @Override
@@ -69,7 +69,7 @@ public class OrderingTest
             return false;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#exists()
          */
         @Override
@@ -78,7 +78,7 @@ public class OrderingTest
             return false;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#getFile()
          */
         @Override
@@ -87,7 +87,7 @@ public class OrderingTest
             return null;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#getInputStream()
          */
         @Override
@@ -96,7 +96,13 @@ public class OrderingTest
             return null;
         }
 
-        /** 
+        @Override
+        public ReadableByteChannel getReadableByteChannel() throws IOException
+        {
+            return null;
+        }
+
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#getName()
          */
         @Override
@@ -104,17 +110,8 @@ public class OrderingTest
         {
             return _name;
         }
-
-        /** 
-         * @see org.eclipse.jetty.util.resource.Resource#getOutputStream()
-         */
-        @Override
-        public OutputStream getOutputStream() throws IOException, SecurityException
-        {
-            return null;
-        }
-
-        /** 
+        
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#getURL()
          */
         @Override
@@ -123,7 +120,7 @@ public class OrderingTest
             return null;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#isContainedIn(org.eclipse.jetty.util.resource.Resource)
          */
         @Override
@@ -132,7 +129,7 @@ public class OrderingTest
             return false;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#isDirectory()
          */
         @Override
@@ -141,7 +138,7 @@ public class OrderingTest
             return false;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#lastModified()
          */
         @Override
@@ -150,7 +147,7 @@ public class OrderingTest
             return 0;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#length()
          */
         @Override
@@ -159,7 +156,7 @@ public class OrderingTest
             return 0;
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#list()
          */
         @Override
@@ -168,15 +165,15 @@ public class OrderingTest
             return null;
         }
 
-        /** 
-         * @see org.eclipse.jetty.util.resource.Resource#release()
+        /**
+         * @see org.eclipse.jetty.util.resource.Resource#close()
          */
         @Override
-        public void release()
+        public void close()
         {
         }
 
-        /** 
+        /**
          * @see org.eclipse.jetty.util.resource.Resource#renameTo(org.eclipse.jetty.util.resource.Resource)
          */
         @Override
@@ -184,9 +181,9 @@ public class OrderingTest
         {
             return false;
         }
-        
+
     }
-    
+
     @Test
     public void testRelativeOrdering0 ()
     throws Exception
@@ -196,7 +193,7 @@ public class OrderingTest
         MetaData metaData = new MetaData();
         List<Resource> resources = new ArrayList<Resource>();
         metaData._ordering = new RelativeOrdering(metaData);
-        
+
         //A: after others, after C
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
@@ -208,7 +205,7 @@ public class OrderingTest
         f1._otherType = FragmentDescriptor.OtherType.After;
         //((RelativeOrdering)metaData._ordering).addAfterOthers(r1);
         f1._afters.add("C");
-        
+
         //B: before others
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
@@ -219,7 +216,7 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar2, f2);
         f2._otherType = FragmentDescriptor.OtherType.Before;
         //((RelativeOrdering)metaData._ordering).addBeforeOthers(r2);
-        
+
         //C: after others
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
@@ -230,29 +227,29 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar3, f3);
         f3._otherType = FragmentDescriptor.OtherType.After;
         //((RelativeOrdering)metaData._ordering).addAfterOthers(r3);
-        
+
         //D: no ordering
         TestResource jar4 = new TestResource("D");
         resources.add(jar4);
         TestResource r4 = new TestResource("D/web-fragment.xml");
         FragmentDescriptor f4 = new FragmentDescriptor(r4);
-        f4._name="D"; 
+        f4._name="D";
         metaData._webFragmentNameMap.put(f4._name, f4);
         metaData._webFragmentResourceMap.put(jar4, f4);
         f4._otherType = FragmentDescriptor.OtherType.None;
         //((RelativeOrdering)metaData._ordering).addNoOthers(r4);
-        
-        //E: no ordering   
-        TestResource jar5 = new TestResource("E");     
+
+        //E: no ordering
+        TestResource jar5 = new TestResource("E");
         resources.add(jar5);
         TestResource r5 = new TestResource("E/web-fragment.xml");
         FragmentDescriptor f5 = new FragmentDescriptor(r5);
-        f5._name="E"; 
+        f5._name="E";
         metaData._webFragmentNameMap.put(f5._name, f5);
         metaData._webFragmentResourceMap.put(jar5, f5);
         f5._otherType = FragmentDescriptor.OtherType.None;
         //((RelativeOrdering)metaData._ordering).addNoOthers(r5);
-        
+
         //F: before others, before B
         TestResource jar6 = new TestResource("F");
         resources.add(jar6);
@@ -264,22 +261,22 @@ public class OrderingTest
         f6._otherType = FragmentDescriptor.OtherType.Before;
         //((RelativeOrdering)metaData._ordering).addBeforeOthers(r6);
         f6._befores.add("B");
-        
+
         //
         // p.70 outcome: F, B, D, E, C, A
         //
         String[] outcomes = {"FBDECA"};
         List<Resource> orderedList = metaData._ordering.order(resources);
-        
+
         String result = "";
         for (Resource r:orderedList)
             result+=(((TestResource)r)._name);
-        
+
         if (!checkResult(result, outcomes))
             fail("No outcome matched "+result);
     }
- 
-  
+
+
 
     @Test
     public void testRelativeOrdering1 ()
@@ -289,7 +286,7 @@ public class OrderingTest
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
         metaData._ordering = new RelativeOrdering(metaData);
-        
+
         //Example from ServletSpec p.70-71
         //No name: after others, before C
         TestResource jar1 = new TestResource("plain");
@@ -297,12 +294,12 @@ public class OrderingTest
         TestResource r1 = new TestResource("plain/web-fragment.xml");
         FragmentDescriptor f1 = new FragmentDescriptor(r1);
         f1._name = FragmentDescriptor.NAMELESS+"1";
-        metaData._webFragmentNameMap.put(f1._name, f1); 
+        metaData._webFragmentNameMap.put(f1._name, f1);
         metaData._webFragmentResourceMap.put(jar1,f1);
         f1._otherType = FragmentDescriptor.OtherType.After;
         //((RelativeOrdering)metaData._ordering).addAfterOthers(f1);
         f1._befores.add("C");
-        
+
         //B: before others
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
@@ -313,7 +310,7 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar2,f2);
         f2._otherType = FragmentDescriptor.OtherType.Before;
         //((RelativeOrdering)metaData._ordering).addBeforeOthers(f2);
-               
+
         //C: no ordering
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
@@ -324,18 +321,18 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar3,f3);
         //((RelativeOrdering)metaData._ordering).addNoOthers(f3);
         f3._otherType = FragmentDescriptor.OtherType.None;
-        
+
         //D: after others
         TestResource jar4 = new TestResource("D");
         resources.add(jar4);
         TestResource r4 = new TestResource("D/web-fragment.xml");
         FragmentDescriptor f4 = new FragmentDescriptor(r4);
-        f4._name="D"; 
+        f4._name="D";
         metaData._webFragmentNameMap.put(f4._name, f4);
         metaData._webFragmentResourceMap.put(jar4,f4);
         //((RelativeOrdering)metaData._ordering).addAfterOthers(f4);
         f4._otherType = FragmentDescriptor.OtherType.After;
-        
+
         //E: before others
         TestResource jar5 = new TestResource("E");
         resources.add(jar5);
@@ -346,7 +343,7 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar5,f5);
         //((RelativeOrdering)metaData._ordering).addBeforeOthers(f5);
         f5._otherType = FragmentDescriptor.OtherType.Before;
-        
+
         //F: no ordering
         TestResource jar6 = new TestResource("F");
         resources.add(jar6);
@@ -357,7 +354,7 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar6,f6);
         //((RelativeOrdering)metaData._ordering).addNoOthers(f6);
         f6._otherType = FragmentDescriptor.OtherType.None;
-        
+
         List<Resource> orderedList = metaData._ordering.order(resources);
 
         // p.70-71 Possible outcomes are:
@@ -367,20 +364,20 @@ public class OrderingTest
         // E, B, F, noname, D, C
         // E, B, F, D, noname, C
         //
-        String[] outcomes = {"BEFplainCD", 
+        String[] outcomes = {"BEFplainCD",
                              "BEFplainDC",
                              "EBFplainCD",
                              "EBFplainDC",
                              "EBFDplain"};
-        
+
         String orderedNames = "";
         for (Resource r:orderedList)
             orderedNames+=(((TestResource)r)._name);
-        
+
         if (!checkResult(orderedNames, outcomes))
             fail("No outcome matched "+orderedNames);
     }
-    
+
     @Test
     public void testRelativeOrdering2 ()
     throws Exception
@@ -389,9 +386,9 @@ public class OrderingTest
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
         metaData._ordering = new RelativeOrdering(metaData);
-        
+
         //Example from Spec p. 71-72
-        
+
         //A: after B
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
@@ -403,7 +400,7 @@ public class OrderingTest
         //((RelativeOrdering)metaData._ordering).addNoOthers(f1);
         f1._otherType = FragmentDescriptor.OtherType.None;
         f1._afters.add("B");
-        
+
         //B: no order
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
@@ -414,7 +411,7 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar2, f2);
         //((RelativeOrdering)metaData._ordering).addNoOthers(f2);
         f2._otherType = FragmentDescriptor.OtherType.None;
-        
+
         //C: before others
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
@@ -431,7 +428,7 @@ public class OrderingTest
         resources.add(jar4);
         TestResource r4 = new TestResource("D/web-fragment.xml");
         FragmentDescriptor f4 = new FragmentDescriptor(r4);
-        f4._name="D"; 
+        f4._name="D";
         metaData._webFragmentNameMap.put(f4._name, f4);
         metaData._webFragmentResourceMap.put(jar4, f4);
         //((RelativeOrdering)metaData._ordering).addNoOthers(f4);
@@ -445,21 +442,21 @@ public class OrderingTest
         String[] outcomes = {"CBDA",
                              "CDBA",
                              "CBAD"};
-       
- 
+
+
         List<Resource> orderedList = metaData._ordering.order(resources);
         String result = "";
         for (Resource r:orderedList)
            result+=(((TestResource)r)._name);
-        
+
         if (!checkResult(result, outcomes))
             fail ("No outcome matched "+result);
     }
-    
+
     @Test
     public void testRelativeOrdering3 ()
     throws Exception
-    { 
+    {
         List<Resource> resources = new ArrayList<Resource>();
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
@@ -489,7 +486,7 @@ public class OrderingTest
         f2._otherType = FragmentDescriptor.OtherType.Before;
         f2._befores.add("C");
 
-        //C: no ordering   
+        //C: no ordering
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
         TestResource r3 = new TestResource("C/web-fragment.xml");
@@ -499,31 +496,31 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar3,f3);
         //((RelativeOrdering)metaData._ordering).addNoOthers(f3);
         f3._otherType = FragmentDescriptor.OtherType.None;
-        
+
         //result: BAC
         String[] outcomes = {"BAC"};
-        
+
         List<Resource> orderedList = metaData._ordering.order(resources);
         String result = "";
         for (Resource r:orderedList)
            result+=(((TestResource)r)._name);
-        
+
         if (!checkResult(result, outcomes))
             fail ("No outcome matched "+result);
     }
-    
+
     @Test
     public void testCircular1 ()
     throws Exception
     {
-        
+
         //A: after B
         //B: after A
         List<Resource> resources = new ArrayList<Resource>();
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
         metaData._ordering = new RelativeOrdering(metaData);
-        
+
         //A: after B
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
@@ -535,7 +532,7 @@ public class OrderingTest
         //((RelativeOrdering)metaData._ordering).addNoOthers(f1);
         f1._otherType = FragmentDescriptor.OtherType.None;
         f1._afters.add("B");
-        
+
         //B: after A
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
@@ -558,18 +555,18 @@ public class OrderingTest
             assertTrue (e instanceof IllegalStateException);
         }
     }
- 
-    
-    
+
+
+
     @Test
     public void testInvalid1 ()
     throws Exception
-    {      
+    {
         List<Resource> resources = new ArrayList<Resource>();
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
         metaData._ordering = new RelativeOrdering(metaData);
-        
+
         //A: after others, before C
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
@@ -581,7 +578,7 @@ public class OrderingTest
         //((RelativeOrdering)metaData._ordering).addAfterOthers(r1);
         f1._otherType = FragmentDescriptor.OtherType.After;
         f1._befores.add("C");
-        
+
         //B: before others, after C
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
@@ -593,7 +590,7 @@ public class OrderingTest
         //((RelativeOrdering)metaData._ordering).addBeforeOthers(r2);
         f2._otherType = FragmentDescriptor.OtherType.Before;
         f2._afters.add("C");
-        
+
         //C: no ordering
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
@@ -604,7 +601,7 @@ public class OrderingTest
         metaData._webFragmentResourceMap.put(jar3,f3);
         //((RelativeOrdering)metaData._ordering).addNoOthers(r3);
         f3._otherType = FragmentDescriptor.OtherType.None;
-        
+
         try
         {
             List<Resource> orderedList = metaData._ordering.order(resources);
@@ -635,7 +632,7 @@ public class OrderingTest
         ((AbsoluteOrdering)metaData._ordering).add("B");
         ((AbsoluteOrdering)metaData._ordering).add("C");
         ((AbsoluteOrdering)metaData._ordering).addOthers();
-        
+
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
         TestResource r1 = new TestResource("A/web-fragment.xml");
@@ -643,7 +640,7 @@ public class OrderingTest
         f1._name = "A";
         metaData._webFragmentNameMap.put(f1._name, f1);
         metaData._webFragmentResourceMap.put(jar1,f1);
-        
+
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
         TestResource r2 = new TestResource("B/web-fragment.xml");
@@ -651,7 +648,7 @@ public class OrderingTest
         f2._name="B";
         metaData._webFragmentNameMap.put(f2._name, f2);
         metaData._webFragmentResourceMap.put(jar2, f2);
-        
+
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
         TestResource r3 = new TestResource("C/web-fragment.xml");
@@ -659,15 +656,15 @@ public class OrderingTest
         f3._name="C";
         metaData._webFragmentNameMap.put(f3._name, f3);
         metaData._webFragmentResourceMap.put(jar3, f3);
-        
+
         TestResource jar4 = new TestResource("D");
         resources.add(jar4);
         TestResource r4 = new TestResource("D/web-fragment.xml");
         FragmentDescriptor f4 = new FragmentDescriptor((Resource)null);
-        f4._name="D"; 
+        f4._name="D";
         metaData._webFragmentNameMap.put(f4._name, f4);
         metaData._webFragmentResourceMap.put(jar4, f4);
-        
+
         TestResource jar5 = new TestResource("E");
         resources.add(jar5);
         TestResource r5 = new TestResource("E/web-fragment.xml");
@@ -675,7 +672,7 @@ public class OrderingTest
         f5._name="E";
         metaData._webFragmentNameMap.put(f5._name, f5);
         metaData._webFragmentResourceMap.put(jar5, f5);
-        
+
         TestResource jar6 = new TestResource("plain");
         resources.add(jar6);
         TestResource r6 = new TestResource ("plain/web-fragment.xml");
@@ -683,32 +680,32 @@ public class OrderingTest
         f6._name=FragmentDescriptor.NAMELESS+"1";
         metaData._webFragmentNameMap.put(f6._name, f6);
         metaData._webFragmentResourceMap.put(jar6, f6);
-        
+
         List<Resource> list = metaData._ordering.order(resources);
-        
+
         String[] outcomes = {"ABCDEplain"};
         String result = "";
         for (Resource r:list)
             result += ((TestResource)r)._name;
-        
+
         if (!checkResult(result, outcomes))
             fail("No outcome matched "+result);
     }
-   
+
     @Test
     public void testAbsoluteOrdering2 ()
     throws Exception
     {
-        // C,B,A 
+        // C,B,A
         List<Resource> resources = new ArrayList<Resource>();
-        
+
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
         metaData._ordering = new AbsoluteOrdering(metaData);
         ((AbsoluteOrdering)metaData._ordering).add("C");
         ((AbsoluteOrdering)metaData._ordering).add("B");
         ((AbsoluteOrdering)metaData._ordering).add("A");
-        
+
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
         TestResource r1 = new TestResource("A/web-fragment.xml");
@@ -716,15 +713,15 @@ public class OrderingTest
         f1._name = "A";
         metaData._webFragmentNameMap.put(f1._name, f1);
         metaData._webFragmentResourceMap.put(jar1,f1);
-        
-        TestResource jar2 = new TestResource("B");  
+
+        TestResource jar2 = new TestResource("B");
         resources.add(jar2);
         TestResource r2 = new TestResource("B/web-fragment.xml");
         FragmentDescriptor f2 = new FragmentDescriptor(r2);
         f2._name="B";
         metaData._webFragmentNameMap.put(f2._name, f2);
         metaData._webFragmentResourceMap.put(jar2,f2);
-        
+
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
         TestResource r3 = new TestResource("C/web-fragment.xml");
@@ -732,15 +729,15 @@ public class OrderingTest
         f3._name="C";
         metaData._webFragmentNameMap.put(f3._name, f3);
         metaData._webFragmentResourceMap.put(jar3,f3);
-        
+
         TestResource jar4 = new TestResource("D");
         resources.add(jar4);
         TestResource r4 = new TestResource("D/web-fragment.xml");
         FragmentDescriptor f4 = new FragmentDescriptor(r4);
-        f4._name="D"; 
+        f4._name="D";
         metaData._webFragmentNameMap.put(f4._name, f4);
         metaData._webFragmentResourceMap.put(jar4,f4);
-        
+
         TestResource jar5 = new TestResource("E");
         resources.add(jar5);
         TestResource r5 = new TestResource("E/web-fragment.xml");
@@ -748,7 +745,7 @@ public class OrderingTest
         f5._name="E";
         metaData._webFragmentNameMap.put(f5._name, f5);
         metaData._webFragmentResourceMap.put(jar5,f5);
-        
+
         TestResource jar6 = new TestResource("plain");
         resources.add(jar6);
         TestResource r6 = new TestResource("plain/web-fragment.xml");
@@ -756,36 +753,36 @@ public class OrderingTest
         f6._name=FragmentDescriptor.NAMELESS+"1";
         metaData._webFragmentNameMap.put(f6._name, f6);
         metaData._webFragmentResourceMap.put(jar6,f6);
-        
+
         List<Resource> list = metaData._ordering.order(resources);
         String[] outcomes = {"CBA"};
         String result = "";
         for (Resource r:list)
             result += ((TestResource)r)._name;
-        
-        
+
+
         if (!checkResult(result, outcomes))
             fail("No outcome matched "+result);
     }
-    
+
     @Test
     public void testAbsoluteOrdering3 ()
     throws Exception
     {
         //empty <absolute-ordering>
-        
+
         WebAppContext wac = new WebAppContext();
         MetaData metaData = new MetaData();
         metaData._ordering = new AbsoluteOrdering(metaData);
         List<Resource> resources = new ArrayList<Resource>();
-        
+
         resources.add(new TestResource("A"));
         resources.add(new TestResource("B"));
-        
+
         List<Resource> list = metaData._ordering.order(resources);
         assertTrue(list.isEmpty());
     }
-    
+
     @Test
     public void testRelativeOrderingWithPlainJars ()
     throws Exception
@@ -820,7 +817,7 @@ public class OrderingTest
         f2._otherType = FragmentDescriptor.OtherType.Before;
         f2._befores.add("C");
 
-        //C: after A   
+        //C: after A
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
         TestResource r3 = new TestResource("C/web-fragment.xml");
@@ -831,15 +828,15 @@ public class OrderingTest
         //((RelativeOrdering)metaData._ordering).addNoOthers(f3);
         f3._otherType = FragmentDescriptor.OtherType.None;
         f3._afters.add("A");
-        
+
         //No fragment jar 1
         TestResource r4 = new TestResource("plain1");
         resources.add(r4);
-        
+
         //No fragment jar 2
         TestResource r5 = new TestResource("plain2");
         resources.add(r5);
-        
+
         //result: BAC
         String[] outcomes = {"Bplain1plain2AC"};
 
@@ -847,11 +844,11 @@ public class OrderingTest
         String result = "";
         for (Resource r:orderedList)
            result+=(((TestResource)r)._name);
-        
+
         if (!checkResult(result, outcomes))
             fail ("No outcome matched "+result);
     }
-    
+
     @Test
     public void testRelativeOrderingWithPlainJars2 ()
     throws Exception
@@ -905,7 +902,7 @@ public class OrderingTest
         ((AbsoluteOrdering)metaData._ordering).add("B");
         ((AbsoluteOrdering)metaData._ordering).add("C");
         ((AbsoluteOrdering)metaData._ordering).addOthers();
-        
+
         TestResource jar1 = new TestResource("A");
         resources.add(jar1);
         TestResource r1 = new TestResource("A/web-fragment.xml");
@@ -913,7 +910,7 @@ public class OrderingTest
         f1._name = "A";
         metaData._webFragmentNameMap.put(f1._name, f1);
         metaData._webFragmentResourceMap.put(jar1,f1);
-        
+
         TestResource jar2 = new TestResource("B");
         resources.add(jar2);
         TestResource r2 = new TestResource("B/web-fragment.xml");
@@ -921,7 +918,7 @@ public class OrderingTest
         f2._name="B";
         metaData._webFragmentNameMap.put(f2._name, f2);
         metaData._webFragmentResourceMap.put(jar2, f2);
-        
+
         TestResource jar3 = new TestResource("C");
         resources.add(jar3);
         TestResource r3 = new TestResource("C/web-fragment.xml");
@@ -929,15 +926,15 @@ public class OrderingTest
         f3._name="C";
         metaData._webFragmentNameMap.put(f3._name, f3);
         metaData._webFragmentResourceMap.put(jar3, f3);
-        
+
         TestResource jar4 = new TestResource("D");
         resources.add(jar4);
         TestResource r4 = new TestResource("D/web-fragment.xml");
         FragmentDescriptor f4 = new FragmentDescriptor((Resource)null);
-        f4._name="D"; 
+        f4._name="D";
         metaData._webFragmentNameMap.put(f4._name, f4);
         metaData._webFragmentResourceMap.put(jar4, f4);
-        
+
         TestResource jar5 = new TestResource("E");
         resources.add(jar5);
         TestResource r5 = new TestResource("E/web-fragment.xml");
@@ -945,7 +942,7 @@ public class OrderingTest
         f5._name="E";
         metaData._webFragmentNameMap.put(f5._name, f5);
         metaData._webFragmentResourceMap.put(jar5, f5);
-        
+
         TestResource jar6 = new TestResource("plain");
         resources.add(jar6);
         TestResource r6 = new TestResource("plain/web-fragment.xml");
@@ -953,27 +950,27 @@ public class OrderingTest
         f6._name=FragmentDescriptor.NAMELESS+"1";
         metaData._webFragmentNameMap.put(f6._name, f6);
         metaData._webFragmentResourceMap.put(jar6, f6);
-        
+
         //plain jar
         TestResource r7 = new TestResource("plain1");
         resources.add(r7);
-        
+
         TestResource r8 = new TestResource("plain2");
         resources.add(r8);
-        
+
         List<Resource> list = metaData._ordering.order(resources);
-        
+
         String[] outcomes = {"ABCDEplainplain1plain2"};
         String result = "";
         for (Resource r:list)
             result += ((TestResource)r)._name;
-        
+
         if (!checkResult(result, outcomes))
             fail("No outcome matched "+result);
     }
-    
-    
-    
+
+
+
     public boolean checkResult (String result, String[] outcomes)
     {
         boolean matched = false;
