@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2016 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2017 Mort Bay Consulting Pty. Ltd.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -18,6 +18,10 @@
 
 package org.eclipse.jetty.server.session;
 
+import java.sql.DriverManager;
+import java.sql.SQLException;
+
+import org.junit.After;
 import org.junit.Test;
 
 /**
@@ -38,4 +42,16 @@ public class ImmortalSessionTest extends AbstractImmortalSessionTest
     {
         super.testImmortalSession();
     }
+    
+    @After
+    public void tearDown() throws Exception 
+    {
+        try
+        {
+            DriverManager.getConnection( "jdbc:derby:sessions;shutdown=true" );
+        }
+        catch( SQLException expected )
+        {
+        }
+    }
 }
